@@ -1,13 +1,13 @@
-# Copyrights 2011-2012 by [Mark Overmeer].
+# Copyrights 2011-2013 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.00.
+# Pod stripped from pm file by OODoc 2.01.
 use warnings;
 use strict;
 
 package XML::Compile::SOAP::WSS;
 use vars '$VERSION';
-$VERSION = '1.07';
+$VERSION = '1.09';
 
 use base 'XML::Compile::SOAP::Extension';
 
@@ -17,12 +17,16 @@ use XML::Compile::WSS::Util  qw/:wss11 :utp11/;
 use XML::Compile::WSS        ();
 use XML::Compile::SOAP::Util qw/SOAP11ENV/;
 
+use Scalar::Util             qw/weaken/;
+
 
 sub init($)
 {   my ($self, $args) = @_;
     $self->SUPER::init($args);
     $self->{XCSW_wss} = [];
+
     my $schema = $self->{XCSW_schema} = $args->{schema};
+    weaken $self->{XCSW_schema};
 
     # [1.0] to support backwards compat
     XML::Compile::WSS->loadSchemas($schema, '1.1') if $schema;
@@ -34,8 +38,10 @@ sub wsdl11Init($$)
     $self->SUPER::wsdl11Init($wsdl, $args);
 
     $self->{XCSW_schema} = $wsdl;
+    weaken $self->{XCSW_schema};
+
     XML::Compile::WSS->loadSchemas($wsdl, '1.1');
-    $wsdl->prefixes('SOAP-ENV' => SOAP11ENV);
+    $wsdl->addPrefixes('SOAP-ENV' => SOAP11ENV);
 
     $self;
 }
@@ -55,22 +61,26 @@ before the WSDL because it influences its interpretation";
 
 sub soap11ClientWrapper($$$)
 {   my ($self, $op, $call, $args) = @_;
-    # Add empty security object, otherwise hooks will not get called.
-    # May get overwritten by user supplied element or sublist of wss's.
     sub {
-        my %data = @_;
-        my $sec  = $data{wsse_Security};
-        return $call->(%data)
+        my $data = @_==1 ? shift : {@_};
+        my $sec  = $data->{wsse_Security};
+
+        # Support pre-1.0 interface
+        return $call->($data)
             if ref $sec eq 'HASH';
 
+        # select plugins
         my $wss  = $sec || $self->{XCSW_wss};
         my @wss  = ref $wss eq 'ARRAY' ? @$wss : $wss;
-        my $secw = $data{wsse_Security} = {};
 
-        my $doc  = $data{_doc} ||= XML::LibXML::Document->new('1.0','UTF-8');
+        # Adding WSS headers to $secw
+        my $secw = $data->{wsse_Security} = {};
+        my $doc  = $data->{_doc} ||= XML::LibXML::Document->new('1.0','UTF-8');
         $_->create($doc, $secw) for @wss;
  
-        my ($answer, $trace) = $call->(%data);
+        # The real work: SOAP message formatting and exchange
+        my ($answer, $trace) = $call->($data);
+
         if(defined $answer)
         {   my $secr = $answer->{wsse_Security} ||= {};
             $_->check($secr) for @wss;
@@ -84,9 +94,13 @@ sub soap11ClientWrapper($$$)
 
 sub schema()   { shift->{XCSW_schema} }
 sub features() { @{shift->{XCSW_wss}} }
+
 sub addFeature($)
-{   my ($wss, $n) = (shift->{XCSW_wss}, shift);
-    push @$wss, $n;
+{   my ($self, $n) = @_;
+    my $schema = $n->schema
+        or error __x"no schema yet. Instantiate ::WSS before ::WSDL";
+
+    push @{$self->{XCSW_wss}}, $n;
     $n;
 }
 
@@ -118,34 +132,16 @@ sub timestamp(%)
 }
 
 
+
 sub signature(%)
 {   my ($self, %args) = @_;
     my $schema = $args{schema} || $self->schema;
+
+    $args{sign_types} ||= 'SOAP-ENV:Body';
+    $args{sign_put}   ||= 'wsse:SecurityHeaderType';
+    $args{sign_when}  ||= 'SOAP-ENV:Envelope';
+
     my $sig    = $self->_start('XML::Compile::WSS::Signature', \%args);
-
-    my $sign_body =
-     +{ type     => 'SOAP-ENV:Body'
-      , after    => sub {
-          my ($doc, $xml) = @_;
-
-          # This is called twice, caused by the trick to get first the
-          # body than the header processed when writing an envelope.
-          # The second time, the signature element is already prepared,
-          # no signing skipped.
-          $sig->signElement($xml, id => 'TheBody');  # returns a fixed elem
-          $sig->createSignature($doc);
-          $xml;
-     }};
-    $schema->declare(WRITER => 'SOAP-ENV:Envelope', hooks => $sign_body);
-
-    my $check_body =
-     +{ type   => 'SOAP-ENV:Body'
-      , before => sub {
-          my ($node, $path) = @_;
-          $sig->checkElement($node);
-      }};
-    $schema->declare(READER => 'SOAP-ENV:Envelope', hooks => $check_body);
-
     $sig;
 }
 
@@ -153,7 +149,7 @@ sub signature(%)
 # [1.0] Expired interface
 sub wsseBasicAuth($$$@)
 {   my ($self, $username, $password, $pwtype, %args) = @_;
-    # use XML::Compile::WSS::BasicAuth!!!  The method will be removed!
+    # use XML::Compile::WSS::BasicAuth!!!  This method will be removed!
 
     eval "require XML::Compile::WSS::BasicAuth";
     panic $@ if $@;
@@ -166,14 +162,14 @@ sub wsseBasicAuth($$$@)
       , schema    => $self->schema
       );
 
-   my $doc  = XML::LibXML::Document->new('1.0', 'UTF-8');
-   $auth->create($doc, {});
+    my $doc  = XML::LibXML::Document->new('1.0', 'UTF-8');
+    $auth->create($doc, {});
 }
 
 # [1.0] Expired interface
 sub wsseTimestamp($$$@)
 {   my ($self, $created, $expires, %args) = @_;
-    # use XML::Compile::WSS::Timestamp!!!  The method will be removed!
+    # use XML::Compile::WSS::Timestamp!!!  This method will be removed!
 
     eval "require XML::Compile::WSS::Timestamp";
     panic $@ if $@;
@@ -185,9 +181,8 @@ sub wsseTimestamp($$$@)
       , schema  => $self->schema
       );
 
-   my $doc  = XML::LibXML::Document->new('1.0', 'UTF-8');
-   $ts->create($doc, {});
+    my $doc  = XML::LibXML::Document->new('1.0', 'UTF-8');
+    $ts->create($doc, {});
 }
- 
 
 1;
